Model subclass:#GraphColumn
	instanceVariableNames:'aspects functionYblock'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-Graphs'
!


!GraphColumn class methodsFor:'instance creation'!

name:aName
    ^ self new name:aName
!

new
    ^ super basicNew initialize
! !

!GraphColumn class methodsFor:'menu definitions'!

colorMenuSelector:aSelector
    "specification used to build the Menu
    "
    ^ ColorMenu colorMenu:true value:aSelector.
!

middleButtonMenu
    "this window spec was automatically generated by the ST/X MenuEditor"

    "do not manually edit this - the builder may not be able to
     handle the specification if its corrupted."

    "
     MenuEditor new openOnClass:GraphColumn andSelector:#middleButtonMenu
     (Menu new fromLiteralArrayEncoding:(GraphColumn middleButtonMenu)) startUp
    "

    <resource: #menu>

    ^
     
       #(#Menu
          
           #(
             #(#MenuItem
                #'label:' 'Style'
                #'argument:' #'lineStyle:'
                #'submenuChannel:' #'styleMenuSelector:'
            )
             #(#MenuItem
                #'label:' 'Size'
                #'argument:' #'lineWidth:'
                #'submenuChannel:' #'widthMenuSelector:'
            )
             #(#MenuItem
                #'label:' 'Color'
                #'argument:' #'foregroundColor:'
                #'submenuChannel:' #'colorMenuSelector:'
            )
             #(#MenuItem
                #'label:' 'Zoom Y'
                #'argument:' #'zoomY:'
                #'submenuChannel:' #'zoomMenuSelector:'
            )
          ) nil
          nil
      )
!

styleMenuSelector:aSelector
    |menu width height item bitmap|

    menu   := Menu new.
    width  := 40.
    height := 10.

    #( #solid #dashed ) do:[:style|
        bitmap := Form width:width height:height depth:1.
        bitmap paint:(Color colorId:1).
        bitmap fillRectangleX:0 y:0 width:width height:height.

        bitmap paint:(Color colorId:0).
        bitmap fillRectangleX:0 y:0 width:width height:height.
        bitmap paint:(Color colorId:1).
        bitmap lineWidth:2.
        bitmap lineStyle:style.
        bitmap displayLineFromX:2 y:(height // 2) toX:width-2 y:(height // 2).
        item := MenuItem labeled:bitmap.
        item value:aSelector.
        item argument:style.
        menu addItem:item.
    ].
  ^ menu

"
(self styleMenuSelector:#lineStyle) startUp
"

!

widthMenuSelector:aSelector
    |menu width item bitmap|

    menu  := Menu new.
    width := 40.

    #( 1 2 3 4 5 6 ) do:[:height|
        bitmap := Form width:width height:height depth:1.
        bitmap paint:(Color colorId:1).
        bitmap fillRectangleX:0 y:0 width:width height:height.

        item := MenuItem labeled:bitmap.
        item value:aSelector.
        item argument:height.
        menu addItem:item.
    ].
    item := MenuItem labeled:'other ..'.
    item value:[:arg :panel||n|
        n := Number fromString:(Dialog request:'size:') onError:nil.

        n notNil ifTrue:[
            panel receiver perform:aSelector with:n
        ]
    ].
    menu addItem:item.
  ^ menu

"
(self widthMenuSelector:#lineWidth) startUp
"

!

zoomMenuSelector:aSelector
    "specification used to build the Menu
    "
    |menu width item bitmap|

    menu  := Menu new.
    width := 40.

    #( 25 50 75 nil 100 nil 150 200 nil ) do:[:zY|
        zY notNil ifTrue:[
            item := MenuItem labeled:(zY printString, ' %' ).
            item value:aSelector.
            item argument:(zY / 100).
        ] ifFalse:[
            item := MenuItem labeled:'-'.
        ].
        menu addItem:item.
    ].
    item := MenuItem labeled:'other ..'.
    item value:[:arg :panel||n|
        n := Number fromString:(Dialog request:'zoom %:') onError:nil.

        n notNil ifTrue:[
            panel receiver perform:aSelector with:(n / 100)
        ]
    ].
    menu addItem:item.
  ^ menu

"
(self zoomMenuSelector:#zoomY) startUp
"

! !

!GraphColumn methodsFor:'accessing'!

name
    "get the name of the column
    "
    ^ aspects at:#name ifAbsent:nil
!

name:aName
    "set the name of the column
    "
    self aspectAt:#name put:aName
!

relativeXaxis
    "get the relative X-axis (y == 0) into the view; 0.5 is centered.
    "
    ^ aspects at:#relativeXaxis ifAbsent:0.5


!

relativeXaxis:aNumberOrNil
    "set the relative X-axis (y == 0) into the view; 0.5 is centered; nil sets
     the axis to the center
    "
    |x|

    aNumberOrNil notNil ifTrue:[
        x := aNumberOrNil isInteger ifTrue:[aNumberOrNil]
                                   ifFalse:[aNumberOrNil asFloat]       "/ no fractions
    ] ifFalse:[
        x := 0.5
    ].

    self aspectAt:#relativeXaxis put:x
!

scaleY
    "get the current y-scale of the column (~~ 0)
    "
    ^ aspects at:#scaleY ifAbsent:1
!

scaleY:aNumberOrNil
    "set the current y-scale of the column to a number or nil (== 1).
    "
    |y|

    aNumberOrNil notNil ifTrue:[
        y := aNumberOrNil isInteger ifTrue:[aNumberOrNil]
                                   ifFalse:[aNumberOrNil asFloat].      "/ no fractions

        y > 0 ifFalse:[y := 1]
    ] ifFalse:[
        y := 1
    ].

    self aspectAt:#scaleY put:y
!

shown
    "returns true if the column is shown; each graph which contains the
     column specification will show the column
    "
    ^ aspects at:#shown ifAbsent:true
!

shown:aState
    "true if the column is shown; each graph which contains the
     column specification will show the column
    "
    self aspectAt:#shown put:aState
!

transY
    "get the current y-transition of the column
    "
    ^ aspects at:#transY ifAbsent:0
!

transY:anIntegerOrNil
    "set the current y-transition of the column to an integer or nil (== 0).
    "
    |y|

    anIntegerOrNil notNil ifTrue:[
        y := anIntegerOrNil isInteger ifTrue:[anIntegerOrNil]
                                     ifFalse:[anIntegerOrNil asFloat rounded]   "/ catch fractions
    ] ifFalse:[
        y := 0
    ].
        
    self aspectAt:#transY put:y
!

zoomY
    "get the current y-zoom factor of the column (~~ 0)
    "
    ^ aspects at:#zoomY ifAbsent:1
!

zoomY:aFactor
    "define a specific zoom factor which will zoom the graph/hlines
     to its relative center defined by relativeXaxis
    "
    |y|

    aFactor notNil ifTrue:[
        y := aFactor isInteger ifTrue:[aFactor]
                              ifFalse:[aFactor asFloat].        "/ no fractions

        y > 0 ifFalse:[y := 1]
    ] ifFalse:[
        y := 1
    ].
    self aspectAt:#zoomY put:y

! !

!GraphColumn methodsFor:'accessing Y function'!

functionYblock
    "get the two arguments block which is used to access Y values from
     start into an array. The block should return the collection of y
     values; see method: #from:into:
    "
    ^ functionYblock
!

functionYblock:something
    "set the two arguments block which is used to access Y values from
     start into an array. The block should return the collection of y
     values; see method: #from:into:
    "
    functionYblock := something.
    self changed.
! !

!GraphColumn methodsFor:'accessing Y values'!

yValueAt:anIndex
    "returns the y value at an index
    "
    ^ (self yValuesStartAt:anIndex into:(Array new:1)) at:1
!

yValuesStartAt:start into:anArray
    "returns the Y values from start upto array size. The collection of y
     values should be returned; can use the 'anArray' or your own
    "
    ^ functionYblock value:start value:anArray
! !

!GraphColumn methodsFor:'accessing graph'!

foregroundColor
    "get the color of the graph; if the color is nil the
     foreground color of the graph is used
    "
    ^ aspects at:#foregroundColor ifAbsent:nil

!

foregroundColor:aColor
    "set the color of the graph; if the color is nil the
     foreground color of the graph is used
    "
    self aspectAt:#foregroundColor put:aColor

!

lineStyle
    "get the style in which the graph is drawn; supported
     styles are #solid or #dashed
    "
    ^ aspects at:#lineStyle ifAbsent:#solid
!

lineStyle:aStyle
    "set the style in which the graph is drawn; supported styles are
     #solid or #dashed. If the style is nil, #solid will be set.
    "
    self aspectAt:#lineStyle put:aStyle
!

lineWidth
    "get the line width of the graph
    "
    ^ aspects at:#lineWidth ifAbsent:1
!

lineWidth:anInteger
    "set the line width of the graph
    "
    self aspectAt:#lineWidth put:(anInteger ? 1)
! !

!GraphColumn methodsFor:'accessing hLines'!

hLineFgColor
    "get the color of the horizontal lines; if the color is nil the
     foreground color of the graph is used
    "
    ^ aspects at:#hLineFgColor ifAbsent:nil
!

hLineFgColor:aColor
    "set the color of the horizontal lines; if the color is nil the
     foreground color of the graph is used
    "
    self aspectAt:#hLineFgColor put:aColor
!

hLineList
    "get list of horizontal lines assigned to the column
    "
    ^ aspects at:#hLineList ifAbsent:nil
!

hLineList:aCollection
    "set list of horizontal lines assigned to the column; scalling and
     transition is used for redrawing the horizontal lines
    "
    |col|

    col := (aCollection size ~~ 0) ifTrue:[aCollection] ifFalse:[nil].

    self aspectAt:#hLineList put:col
!

hLineStyle
    "get the style in which the horizontal lines are drawn; supported
     styles are #solid or #dashed.
    "
    ^ aspects at:#hLineStyle ifAbsent:#solid
!

hLineStyle:aStyle
    "set the style in which the horizontal lines are drawn; supported styles
     are #solid or #dashed. If the style is nil, #solid will be set.
    "
    self aspectAt:#hLineStyle put:aStyle
!

hLineWidth
    "get the line width of the horizontal lines
    "
    ^ aspects at:#hLineWidth ifAbsent:1

!

hLineWidth:anInteger
    "set the line width of the horizontal lines
    "
    self aspectAt:#hLineWidth put:(anInteger ? 1)


! !

!GraphColumn methodsFor:'accessing menu'!

middleButtonMenu
    |menu|

    menu := Menu new fromLiteralArrayEncoding:(self class middleButtonMenu).
    menu receiver:self.
  ^ menu


! !

!GraphColumn methodsFor:'aspects'!

aspect
    "returns current aspects
    "
    ^ aspects
!

aspectAt:aKey put:aValue
    |oldValue|

    oldValue := aspects at:aKey ifAbsent:nil.

    oldValue ~= aValue ifTrue:[
        aValue isNil ifTrue:[aspects removeKey:aKey]
                    ifFalse:[aspects at:aKey put:aValue].

        self changed:aKey with:oldValue
    ]
!

aspects:aDictionaryOrNil
    "change aspects; raise a change notification when changed. If the argument is
     nil, the default aspects are set
    "
    |oldAspects|

    aDictionaryOrNil == aspects ifFalse:[
        oldAspects := aspects.
        self setAspects:aDictionaryOrNil.
        self changed:#aspects with:oldAspects.
    ]
!

setAspects:aDictionaryOrNil
    "change aspects; raise no notification. If the argument is
     nil, the default aspects are set
    "
    aDictionaryOrNil isNil ifTrue:[
        aspects := IdentityDictionary new.

        aspects at:#hLineWidth    put:1.
        aspects at:#lineWidth     put:1.
        aspects at:#scaleY        put:1.
        aspects at:#zoomY         put:1.
        aspects at:#transY        put:0.
        aspects at:#relativeXaxis put:0.5.
    ] ifFalse:[
        aspects := aDictionaryOrNil
    ]
! !

!GraphColumn methodsFor:'initialization'!

initialize
    "setup default values
    "
    super initialize.
    self setAspects:nil.
! !

!GraphColumn methodsFor:'printing'!

printString
    "returns my printable string
    "
    |name|

    ^ (name := self name) notNil ifTrue:[name printString] ifFalse:['']
! !

!GraphColumn methodsFor:'queries'!

centerGraphMaxY:maxY minY:minY
    "set transition Y for the graph dependent on current scaleY; the graph is centered
     to its relative X-Axis (changes transition Y)
    "
    self transY:(((maxY + minY) / 2) * self scaleY)
!

scaleToHeight:aHeight maxY:maxY minY:minY
    "set scaleY dependent on a max and min y value to fit into
     a height >= 1
    "
    |dltY|

    (dltY := (maxY - minY) abs) = 0 ifTrue:[dltY := 1].
    self scaleY:((aHeight max:1) / dltY)
! !

!GraphColumn class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/GraphColumn.st,v 1.1 1998-02-07 15:15:30 ca Exp $'
! !
